import { IGetAllCertificatesOptions } from '@energyweb/issuer-api';
import { QueryBus } from '@nestjs/cqrs';
import { Queue } from 'bull';
import { BlockchainAction, ICertificate } from './types';
import { IClaimCommand, IIssueCommandParams, IIssuedCertificate, ITransferCommand } from '../types';
import { BatchClaimActionResult, BatchIssuanceActionResult, BatchTransferActionResult, ClaimActionResult, IssuanceActionResult, TransferActionResult } from './blockchain-actions.processor';
export declare class OnChainCertificateService<T = null> {
    private readonly queryBus;
    private readonly blockchainActionsQueue;
    constructor(queryBus: QueryBus, blockchainActionsQueue: Queue<BlockchainAction>);
    getAll(options?: IGetAllCertificatesOptions): Promise<ICertificate<T>[]>;
    getById(id: number): Promise<ICertificate<T> | null>;
    issueWithTxHash(params: IIssueCommandParams<T>): Promise<IssuanceActionResult<T>>;
    issue(params: IIssueCommandParams<T>): Promise<IIssuedCertificate<T>>;
    claimWithTxHash(command: IClaimCommand): Promise<ClaimActionResult>;
    claim(command: IClaimCommand): Promise<void>;
    transferWithTxHash(command: ITransferCommand): Promise<TransferActionResult>;
    transfer(command: ITransferCommand): Promise<void>;
    batchIssueWithTxHash(originalCertificates: IIssueCommandParams<T>[]): Promise<BatchIssuanceActionResult>;
    batchIssue(originalCertificates: IIssueCommandParams<T>[]): Promise<number[]>;
    batchClaimWithTxHash(command: IClaimCommand[]): Promise<BatchClaimActionResult>;
    batchClaim(command: IClaimCommand[]): Promise<void>;
    batchTransferWithTxHash(command: ITransferCommand[]): Promise<BatchTransferActionResult>;
    batchTransfer(command: ITransferCommand[]): Promise<void>;
    private mapCertificate;
    private static waitForJobResult;
}
