"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CertificateForUnitTestsService = void 0;
const ethers_1 = require("ethers");
const common_1 = require("@nestjs/common");
let CertificateForUnitTestsService = class CertificateForUnitTestsService {
    constructor() {
        this.serial = 0;
        this.db = [];
    }
    async getAll(options = {}) {
        const lastDate = new Date('2030-01-01T00:00:00.000Z');
        const generationEndFrom = options.generationEndFrom ?? new Date(0);
        const generationEndTo = options.generationEndTo ?? lastDate;
        const generationStartFrom = options.generationStartFrom ?? new Date(0);
        const generationStartTo = options.generationStartTo ?? lastDate;
        const creationTimeFrom = options.creationTimeFrom ?? new Date(0);
        const creationTimeTo = options.creationTimeTo ?? lastDate;
        const deviceId = options.deviceId;
        return this.db.filter((entry) => {
            const isDateOk = new Date(entry.generationStartTime * 1000) >= generationStartFrom &&
                new Date(entry.generationStartTime * 1000) <= generationStartTo &&
                new Date(entry.generationEndTime * 1000) >= generationEndFrom &&
                new Date(entry.generationEndTime * 1000) <= generationEndTo &&
                new Date(entry.creationTime * 1000) >= creationTimeFrom &&
                new Date(entry.creationTime * 1000) <= creationTimeTo;
            const isDeviceOk = deviceId ? deviceId === entry.deviceId : true;
            return isDateOk && isDeviceOk;
        });
    }
    async getById(id) {
        return this.db.find((c) => c.id === id) ?? null;
    }
    async issue(params) {
        this.serial += 1;
        const certificate = {
            claims: [],
            claimers: {},
            creationBlockHash: '',
            creationTime: Math.floor(Date.now() / 1000),
            deviceId: params.deviceId,
            generationStartTime: Math.floor(params.fromTime.getTime() / 1000),
            generationEndTime: Math.floor(params.toTime.getTime() / 1000),
            id: this.serial,
            issuedPrivately: false,
            metadata: params.metadata,
            owners: {
                [params.toAddress]: params.energyValue
            }
        };
        this.db.push(certificate);
        return {
            ...certificate,
            issuedPrivately: false,
            isClaimed: false,
            isOwned: true,
            myClaims: [],
            energy: {
                claimedVolume: '0',
                privateVolume: '0',
                publicVolume: params.energyValue
            }
        };
    }
    async claim(command) {
        const certificate = this.db.find((c) => c.id === command.certificateId);
        if (!certificate) {
            return;
        }
        const value = command.energyValue ??
            Object.values(certificate.owners)
                .reduce((sum, v) => sum.add(v), ethers_1.BigNumber.from(0))
                .toString();
        certificate.claims.push({
            claimData: command.claimData,
            value,
            topic: '0',
            from: command.forAddress,
            id: 0,
            to: command.forAddress
        });
        const ownedValue = certificate.owners[command.forAddress];
        certificate.owners[command.forAddress] = ethers_1.BigNumber.from(ownedValue)
            .sub(ethers_1.BigNumber.from(value))
            .toString();
        certificate.claimers[command.forAddress] = value;
    }
    async transfer(command) {
        const certificate = this.db.find((c) => c.id === command.certificateId);
        if (!certificate) {
            return;
        }
        const value = Number(command.energyValue ?? certificate.owners[command.fromAddress]);
        certificate.owners[command.fromAddress] = (Number(certificate.owners[command.fromAddress]) - value).toString();
        certificate.owners[command.toAddress] = (Number(certificate.owners[command.toAddress] ?? 0) + value).toString();
    }
    async batchIssue(originalCertificates) {
        if (originalCertificates.length === 0) {
            return [];
        }
        const result = await Promise.all(originalCertificates.map((certificate) => this.issue(certificate)));
        return result.map((r) => r.id);
    }
    async batchClaim(command) {
        if (command.length === 0) {
            return;
        }
        await Promise.all(command.map((claim) => this.claim(claim)));
    }
    async batchTransfer(command) {
        if (command.length === 0) {
            return;
        }
        await Promise.all(command.map((transfer) => this.transfer(transfer)));
    }
    async batchIssueWithTxHash(command) {
        return { certificateIds: await this.batchIssue(command), transactionHash: 'txHash' };
    }
    async batchTransferWithTxHash(command) {
        await this.batchTransfer(command);
        return { transactionHash: 'txHash' };
    }
    async batchClaimWithTxHash(command) {
        await this.batchClaim(command);
        return { transactionHash: 'txHash' };
    }
    async issueWithTxHash(command) {
        await this.issue(command);
        return {
            certificate: this.db.find((c) => c.deviceId === command.deviceId),
            transactionHash: 'txHash'
        };
    }
    async transferWithTxHash(command) {
        await this.transfer(command);
        return { transactionHash: 'txHash' };
    }
    async claimWithTxHash(command) {
        await this.claim(command);
        return { transactionHash: 'txHash' };
    }
};
CertificateForUnitTestsService = __decorate([
    common_1.Injectable()
], CertificateForUnitTestsService);
exports.CertificateForUnitTestsService = CertificateForUnitTestsService;
//# sourceMappingURL=onchain-certificateForUnitTests.service.js.map